package aa.display;

import java.awt.event.ActionListener;
import aa.display.FifteenPanel;
import javax.swing.JFrame;
import aa.node.NodeState;
import java.util.HashMap;
import java.util.TreeSet;
import java.util.Stack;
import aa.node.Heuristic;
import aa.node.SearchNode;
import aa.node.Searcher;
import java.awt.*;
import javax.swing.*;


import aa.mynode.FifteenNode;

/**
 * This class "puts it all together".
 * It is the "driver" that uses the other classes to implement a usable
 * fifteen puzzle.
 * 
 * @author: Darius Cooper
 */
public class Puzzle extends JApplet implements aa.display.ValidUserMoveListener {


	//Color for the text
	private static final Color COLOR_TEXT = new Color(25, 50, 20);
	//Color to use for the command buttons (RHS panel)
	private static final Color COLOR_COMMANDS = new Color(150, 200, 255);

	//Strictly, we don't require our own button class, but I intend to
	//add more interesting functionality to this class in a later release.
	private class CommandButton extends javax.swing.JButton {
	    CommandButton() {
		    super();
		    setBackground(COLOR_COMMANDS);
	    }
	    CommandButton(String text) {
		    super(text);
		    setBackground(COLOR_COMMANDS);
		    setBorder(BorderFactory.createRaisedBevelBorder());
	    }
	}
	//The <code>Searcher</code> used to find a solution to the puzzle.
	Searcher searcher = null;

	//A list of SearchNodes representing the solution found by the search
	private SearchNode[] searchNodes = null;
	//A counter showing the step/move number in the solution path 
	int currentStep = 0;
	//The starting (shuffled) state of the puzzle
	SearchNode startNode = null;
	//The heuristic to be used
	Heuristic heuristic = new aa.mynode.FifteenHeuristic();
	//A stack that holds moves made by the user. This is used to UN-do a move
	Stack playerMovesStack;

	//A counter showing the step/move number; it is incremented with each move
	int playerMoveIdx = 0;

	// Various display components 
	JFrame frame = new JFrame();
	JPanel mainPanel = new JPanel();
	JPanel fifteenPanelHolder = new JPanel(new BorderLayout());
	Box mainHBox = Box.createHorizontalBox();
	JPanel commandPanel = new JPanel(new GridLayout(8, 1));
	FifteenPanel fifteenPanel = new FifteenPanel();

	Box movesHBox = Box.createHorizontalBox();
	Box movesVBox = Box.createVerticalBox();
	JLabel movesLabel = new JLabel("Moves: ");
	JLabel movesCountLabel = new JLabel("0");
	Box resetHBox = Box.createHorizontalBox();
	CommandButton resetCountButton = new CommandButton("Reset Counter");

	Box shuffleHBox = Box.createHorizontalBox();
	CommandButton shuffleButton = new CommandButton("Shuffle");
	Box solveHBox = Box.createHorizontalBox();
	CommandButton solveButton = new CommandButton("Solve");
	CommandButton nextStepButton = new CommandButton("Next Step >>");
	CommandButton undoButton = new CommandButton("Undo");
/**
 * Constructor. If using a main() , call <code>init()</code> after constructing
 * the <code>Puzzle</code>object.
 */
public Puzzle() {
	super();
}
/**
 * Handles the users's request to see the next stepm in the solution.
 */
protected void handleNextStep() {
	currentStep++;
	if (currentStep < searchNodes.length) {
		movesCountLabel.setText(Integer.toString(currentStep));
		fifteenPanel.update(new FifteenNode((FifteenNode) searchNodes[currentStep].getNodeState()));
		if (currentStep == searchNodes.length - 1) {
			nextStepButton.setEnabled(false);

			JOptionPane.showMessageDialog(
				frame,
				"Solved!",
				"Solved!",
				JOptionPane.INFORMATION_MESSAGE);
			fifteenPanel.setDisplayOnly(false);
			frame.invalidate();
			frame.repaint();
		}

	}
}
/**
 * Handles the users's request reset the counter that is being displayed.
 * It does not reset the state of the puzzle itself (currently, that can be done
 * by a series of UN-dos). 
 */
protected void handleResetCount() {

	undoButton.setEnabled(false);
	movesCountLabel.setText("0");
	playerMoveIdx=0;
	playerMovesStack = new Stack();
	playerMovesStack.push(new FifteenNode(fifteenPanel.getNodeState()));
}
/**
 * Handles the users's request to shuffle the puzzle. It creates a new start-state
 * This means that a new computer-figured solution is also computed.
 */
protected void handleShuffle() {

	if ((searcher != null) && (searcher.isSolving())) {
		searcher.setStopRequested(true);
		try {
			Thread.sleep(1000);
		} catch (InterruptedException ex) {
		}
		JOptionPane.showMessageDialog(
			frame,
			"Please wait....  closing previous solution",
			"Closing...",
			JOptionPane.INFORMATION_MESSAGE);
		try {
			Thread.sleep(1000);
		} catch (InterruptedException ex) {
		}
	}
	while  ((searcher != null) && (searcher.isSolving())) {
		try {
			Thread.sleep(1000);
		} catch (InterruptedException ex) {
		}
	}
	setUpInitialState();

	solveButton.setEnabled(true);
	nextStepButton.setEnabled(false);

	handleResetCount();
	frame.invalidate();
	frame.repaint();

}
/**
 * Handles the users's request to see the solution.
 * It restores the start-state of the puzzle. The user can then step
 * through the solution with the "Next Step" button.
 */
protected void handleSolve() {

	handleResetCount();
	currentStep = 0;

	if ((searcher != null) && (searcher.isSolving())) {
		JOptionPane.showMessageDialog(
			frame,
			"Please wait....  solving puzzle",
			"Solving...",
			JOptionPane.INFORMATION_MESSAGE);
	}
	while  ((searcher != null) && (searcher.isSolving())) {
		try {
			Thread.sleep(1000);
		} catch (InterruptedException ex) {
		}
	}

	searchNodes = searcher.getSolutionNodes();
	JOptionPane.showMessageDialog(
		frame,
		"Starting from initial state.\nThen, use <NextStep> to walk through the solution",
		"Starting!",
		JOptionPane.INFORMATION_MESSAGE);
	fifteenPanel.setDisplayOnly(true);
	fifteenPanel.update((FifteenNode) searchNodes[currentStep].getNodeState());
	solveButton.setEnabled(false);
	nextStepButton.setEnabled(true);
	frame.invalidate();
	frame.repaint();
}
/**
 * Handles the users's request to undo a user move.
 * While undoing the move, the count is also decremented, to truly allow
 * the user to "take back" the move and not have it be counted as part
 * of the attempted moves.
 */
protected void handleUndo() {

	playerMoveIdx--;
	if (playerMovesStack.size() >= 2) {
		movesCountLabel.setText(Integer.toString(playerMoveIdx));
		playerMovesStack.pop(); //Discard top state
		fifteenPanel.update(new FifteenNode((FifteenNode) playerMovesStack.peek()));		
	}
	if (playerMoveIdx <= 0) {
		undoButton.setEnabled(false);
	}
}
/**
 * Initializes the state of the puzzle and then sets up the display.
 */
public void init() {

	setUpInitialState();

	resetCountButton.setBackground(COLOR_COMMANDS);
	resetCountButton.addActionListener(new ActionListener() {
		public void actionPerformed(java.awt.event.ActionEvent e) {
			handleResetCount();
		}
	});
	shuffleButton.addActionListener(new ActionListener() {
		public void actionPerformed(java.awt.event.ActionEvent e) {
			handleShuffle();
		}
	});
	solveButton.addActionListener(new ActionListener() {
		public void actionPerformed(java.awt.event.ActionEvent e) {
			handleSolve();
		}
	});
	nextStepButton.addActionListener(new ActionListener() {
		public void actionPerformed(java.awt.event.ActionEvent e) {
			handleNextStep();
		}
	});
	undoButton.addActionListener(new ActionListener() {
		public void actionPerformed(java.awt.event.ActionEvent e) {
			handleUndo();
		}
	});
	handleResetCount();

	//resetCountButton.setMinimumSize(new Dimension(200, 50));
	movesLabel.setForeground(COLOR_TEXT);
	movesCountLabel.setForeground(COLOR_TEXT);
	movesHBox.add(Box.createHorizontalStrut(5));
	movesHBox.add(movesLabel);
	movesHBox.add(movesCountLabel);
	movesVBox.add(movesHBox);
	resetHBox.add(resetCountButton);
	resetHBox.add(Box.createHorizontalGlue());
	movesVBox.add(resetHBox);

	//shuffleButton.setMinimumSize(new Dimension(200, 50));
	shuffleHBox.add(shuffleButton);
	shuffleHBox.add(Box.createHorizontalGlue());

	//solveButton.setMinimumSize(new Dimension(200, 50));
	solveHBox.add(solveButton);
	solveHBox.add(Box.createHorizontalStrut(5));
	//nextStepButton.setMinimumSize(new Dimension(200, 50));
	solveHBox.add(nextStepButton);
	nextStepButton.setEnabled(false);
	solveHBox.add(Box.createHorizontalGlue());

	undoButton.setEnabled(false);
	commandPanel.setBackground(COLOR_COMMANDS);
	commandPanel.add(movesHBox);
	commandPanel.add(resetCountButton);
	commandPanel.add(undoButton);
	commandPanel.add(shuffleButton);
	commandPanel.add(solveButton);
	commandPanel.add(nextStepButton);
	commandPanel.add(new JPanel());
	commandPanel.add(new JPanel());

	fifteenPanelHolder.add(fifteenPanel);

	mainHBox.add(Box.createHorizontalStrut(2));
	mainHBox.add(fifteenPanelHolder);
	fifteenPanel.addValidUserMoveListener(this);
	mainHBox.add(Box.createHorizontalStrut(3));
	mainHBox.add(commandPanel);
	mainHBox.add(Box.createHorizontalStrut(2));
	mainHBox.add(Box.createHorizontalGlue());

	mainPanel.setBackground(COLOR_MAIN_BACKGROUND);
	mainPanel.setLayout(new BorderLayout());
	mainPanel.add(mainHBox);

	frame.addWindowListener(new java.awt.event.WindowAdapter() {
		public void windowClosing(java.awt.event.WindowEvent evt) {
			if ((searcher != null) && (searcher.isSolving())) {
				searcher.setStopRequested(true);
			}
		}
	});

	frame.getContentPane().add(mainPanel);
	frame.setSize(600, 400);
	frame.setVisible(true);
	frame.show();

	JOptionPane.showMessageDialog(
		frame,
		"Click on a block to move it.",
		"Try to solve this...",
		JOptionPane.INFORMATION_MESSAGE);
}
/**
 * Creates a <code>Puzzle</code> object and runs it.
 */
public static void main(String[] args) {

	Puzzle puzzle = new Puzzle();

	puzzle.init();

	
}
/**
 * Sets up the initial stat of the puzzle. This involves creating a new
 * <code>NodeState</code> and spawning a search for a solution (in a separate
 * <code>Thread</code>.
 */
protected void setUpInitialState() {

	FifteenNode testState = new FifteenNode();
	startNode = new SearchNode(new FifteenNode(testState));
	fifteenPanel.update(new FifteenNode(testState));
	fifteenPanel.setDisplayOnly(false);

	//If we already have a search thread, we want to re-use it
	//This is particulary important when this runs as an Applet, so that
	//it does not have to have permission for change the ThreadGroup
	if (searcher == null) {
		searcher = new Searcher(startNode, heuristic);
		searcher.setDaemon(true);
		searcher.start();
	} else {
		searcher.setStartNode(startNode);
		searcher.solveFromNewStartNode();
	}

}
/**
 * Invoked when a user makes a valid move in the <code>FifteenPanel</code>.
 * It ensures that the other elements in the display (e.g. counters and buttons
 * are modified in accordance with the new state of the puzzle)
 *
 * @param event A ValidMoveEvent trigger by a user clicking on a button.
 */
public void validUserMovePerformed(aa.display.ValidUserMoveEvent event) {

	playerMoveIdx++;
	movesCountLabel.setText(Integer.toString(playerMoveIdx));
	playerMovesStack.push(new FifteenNode(fifteenPanel.getNodeState()));
	undoButton.setEnabled(true);

	if (fifteenPanel.getNodeState().isGoalState()) {
		JOptionPane.showMessageDialog(
			frame,
			"You did it!",
			"Congrats!",
			JOptionPane.INFORMATION_MESSAGE);
	}
	frame.invalidate();
	frame.repaint();
}

	//Color to use for the main panel background
	private static final Color COLOR_MAIN_BACKGROUND = new Color(50, 115, 115);
}